﻿using Microsoft.VisualStudio.TestTools.WebTesting;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.WebLoadTest.Utilities
{
    [DisplayName("Arithmetic")]
    [Description("Add, subtract, multiply, or divide two integer values.")] 
    public class MathExtractionRule : ExtractionRule
    {
        public MathExtractionRule()
        {
            _ContextValueProvider = null;
        }

        private IContextValueProvider _ContextValueProvider;

        public MathExtractionRule(IContextValueProvider provider)
        {
            if (provider == null)
                throw new ArgumentNullException("provider", "provider is null.");

            _ContextValueProvider = provider;
        }

        private void SetContextValue(string key, string value, ExtractionEventArgs e)
        {
            if (e == null && _ContextValueProvider != null)
            {
                _ContextValueProvider.SetValue(key, value);
            }
            else
            {
                if (e.WebTest.Context.ContainsKey(key) == true)
                {
                    e.WebTest.Context.Remove(key);
                }

                e.WebTest.Context.Add(key, value);
            }
        }

        [DisplayName("Value 1")]
        [Description("Value 1 for the operation.")]
        public string Value1 { get; set; }
        
        [DisplayName("Arithmetic operation")]
        [Description("Valid values are +, -, *, and /.  The arithmetic operation to be performed using value 1 and value 2.")]
        public string Operation { get; set; }

        private MathExtractionRuleOperations MathOperation { get; set; }

        [DisplayName("Value 2")]
        [Description("Value 2 for the operation.")]
        public string Value2 { get; set; }

        [DisplayName("Invalid values become zero")]
        [Description("If this value is set to false, an exception will be thrown.  Otherwise, if value 1 or value 2 is not an integer, replace the value with 0.")]
        public bool InvalidValuesBecomeZero { get; set; }

        private void PopulateOperation()
        {
            if (Operation == "+")
            {
                MathOperation = MathExtractionRuleOperations.Add;
            }
            else if (Operation == "-")
            {
                MathOperation = MathExtractionRuleOperations.Subtract;
            }
            else if (Operation == "*")
            {
                MathOperation = MathExtractionRuleOperations.Multiply;
            }
            else if (Operation == "/")
            {
                MathOperation = MathExtractionRuleOperations.Divide;
            }
            else
            {
                throw new InvalidOperationException("Operation value is not valid.");
            }
        }
        public override void Extract(object sender, ExtractionEventArgs e)
        {
            PopulateOperation();
            int value1AsInt32 = GetValueAsInt32(Value1, "Value 1");
            int value2AsInt32 = GetValueAsInt32(Value2, "Value 2");
            int result = 0;

            if (MathOperation == MathExtractionRuleOperations.Add)
            {
                result = value1AsInt32 + value2AsInt32;
            }
            else if (MathOperation == MathExtractionRuleOperations.Subtract)
            {
                result = value1AsInt32 - value2AsInt32;
            }
            else if (MathOperation == MathExtractionRuleOperations.Multiply)
            {
                result = value1AsInt32 * value2AsInt32;
            }
            else if (MathOperation == MathExtractionRuleOperations.Divide)
            {
                result = value1AsInt32 / value2AsInt32;
            }

            SetContextValue(ContextParameterName, result.ToString(), e);
        }

        private int GetValueAsInt32(string value, string valueName)
        {
            int temp = 0;

            if (Int32.TryParse(value, out temp) == true)
            {
                // good
            }
            else
            {
                if (InvalidValuesBecomeZero == true)
                {
                    temp = 0;
                }
                else
                {
                    throw new ArgumentException(
                        String.Format(
                        "{0}'s value ('{1}') is not an integer.", valueName, value));
                }
            }

            return temp;
        }
    }
}